# bekpip_v0_5.py
# 
# Remark:
# - The program creates an Barker object. 
#   It contains an array of wheels. 
#   The size of each wheel is given by parameters

import sys
import string
import copy
import getopt
import pickle
from Barker import *
from bib_util import *

#======================================
def lecFreq(aFile):
    f = open(aFile,"r")
    ch = f.readline()
    tb = ch.split(",")
    return map(int,tb)

# ---- manage arguments
def usage():
    print "usage:"
    print "Syntax : barker_tui.py [options...]"
    print "Options:"
    print "  -h            Print this Help"
    print "  -c crypto     The cryptogram"
    print "  -f freq       The file which contains Frequencies of "
    print "                letters (in reverse order: Z,Y,X,...B,A)"
    print "  -W wheels     The Wheels (by default: 26:25:23:21:19:17)"
    print "  -T Threhold   Threshold (by default: 10 [10%])"
    print "  -D            Active Debug mode"
try:
  opts, args = getopt.getopt(sys.argv[1:],
    "hc:W:f:T:D", ["help", "crypto=", "wheels=", "freqs=",
    "threshold=", "debug" ])
except getopt.GetoptError as err:
  # print help information and exit:
  print sys.argv[0],": ", str(err) # option -a not recognized"
  usage()
  sys.exit(2)

WHEELS = "26:25:23:21:19:17"
THRESHOLD = 0.10
DEBUG = False
c = Barker( DEBUG)

for o, a in opts:
  if   o in ("-D", "--debug"):
    DEBUG = True
    c.setDebug(DEBUG)
  elif o in ("-h", "--help"):
    usage()
    sys.exit()
  elif o in ("-c", "--crypto"):
    FILE = a
  elif o in ("-W", "--wheels"):
    c.setWheels( a )
  elif o in ("-f", "--freqs"):
    FREQ_FILE = a
    Barker.freq_plain = lecFreq(FREQ_FILE)
  elif o in ("-T", "--threshold"):
    THRESHOLD = a / 100.
  else:
    assert False, "unhandled option"

#=========== Start of Program =====================
try: 
    FILE
    FREQ_FILE
except:
    usage()
    sys.exit()
crypto = litfic( FILE )
c.setCrypto( crypto )
c.setThresHold( float(THRESHOLD) )
tb = []
unchi = []
ch = ""
NUM_WHEEL = 0
# ---- big loop
while 1:
    if ch == "": 
        print "1  - Change Wheel number (",NUM_WHEEL," => ",
        print c.Wheels[ NUM_WHEEL ].getSize(), "), "
        print "     Number of wheels: ", len(c.Wheels),
        print " ThresHold: ", (c.ThresHold*100) , " % "
        print "     Crypto: ", c.crypto[0:50], "..."
        print "2  - Calculate Frequencies of the current Wheel"
        print "3  - Print Frequencies of the current Wheel"
        print "4  - Print Chi2 of the current Wheel"
        print "5  - Create the array of Pins"
        print "6  - Print the array of Pins"
        print "7  - Frequencies with 2 wheels"
        print "9  - Calculate shift between X and Y (equal to kicks of the wheel)"
        print "10 - Input a X/Y/? Pins configuration "
        print "11 - Calculate Cage"
        print "12 - Print Pins configuration (X:Y:?:...) of all wheels"
        print "13 - Save Pins configuration"
        print "14 - Restore Pins configuration"
        print "99 - END"
        ch = 0
        continue
    ch = raw_input("? ")
    choix = ch

    if choix == "1":
       for i in range(len(c.Wheels)):
            aSize = c.Wheels[i].getSize()
            print "%d [%2d], " % (i+1,aSize),
       print
       NUM_WHEEL = int(raw_input("Wheel Number ? ")) - 1
    if choix == "2":
        c.calFreq( NUM_WHEEL )
    if choix == "3":
        c.impFreq( NUM_WHEEL )
    if choix == "4":
        c.impChi2( NUM_WHEEL )
    if choix == "5":
        c.calPinArray( NUM_WHEEL )
    if choix == "6":
        c.imprPinXY( NUM_WHEEL )
    if choix == "7":
       for i in range(len(c.Wheels)):
            aSize = c.Wheels[i].getSize()
            print "%d [%2d], " % (i+1,aSize),
       print " , Current Wheel Number: ", NUM_WHEEL + 1
       OTHER_WHEEL = int(raw_input("Other Wheel Number ? ")) - 1
       c.calFreq2W( NUM_WHEEL, OTHER_WHEEL )     
    if choix == "9":
       if "X" in c.Wheels[NUM_WHEEL].Freq:
           tba = c.Wheels[NUM_WHEEL].Freq["X"]
           tbb = c.Wheels[NUM_WHEEL].Freq["Y"]
           theShift( tba, tbb , c.DEBUG)
    if choix == "10":
       c.modPinXY( NUM_WHEEL )
    if choix == "11":
       c.calCage()
    if choix == "12":
       for i in range(len(c.Wheels)):
            c.imprPinXY(i)
    if choix == "13":
       pickle.dump(c, open("barker.bck", "wb"))
    if choix == "14":
       try:
            c = pickle.load(open("barker.bck", "rb"))
       except:
            print "ERROR: restore failled"
    if choix == "99":
       break

